"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _ejs = _interopRequireDefault(require("ejs"));

var _yeomanGenerator = _interopRequireDefault(require("yeoman-generator"));

var r = _interopRequireWildcard(require("../constants/regex"));

var t = _interopRequireWildcard(require("../constants/templateStrings"));

var _utils = require("../utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const PATH_PREFIX = 'actions';
const CREATORS_FILE_NAME = 'creators';

class ActionGenerator extends _yeomanGenerator.default {
  constructor(args, options) {
    super(args, options);

    _defineProperty(this, "filesToWrite", []);

    _defineProperty(this, "templateConfig", {});

    _defineProperty(this, "indent", void 0);

    _defineProperty(this, "destDir", void 0);

    _defineProperty(this, "destPath", void 0);

    _defineProperty(this, "topLevelPath", void 0);

    _defineProperty(this, "absolutePath", void 0);

    _defineProperty(this, "fileName", void 0);

    _defineProperty(this, "withActionType", true);

    this.indent = options.indent;
    this.destDir = options.destDir;
    this.destPath = this.destDir.join('/');
    this.topLevelPath = `${options.srcDir}/${PATH_PREFIX}`;
    this.absolutePath = `${this.topLevelPath}/${this.destPath}`;
    this.fileName = options.fileName;
  }

  async prompting() {
    const answer = await this.prompt({
      type: 'confirm',
      name: 'withActionType',
      message: 'Create an action type with the same name?'
    });
    this.withActionType = answer.withActionType;
  }

  configuring() {
    this.templateConfig = {
      ACTION_NAME: this.fileName,
      ACTION_TYPE: 't.' + (this.withActionType ? this.fileName.toConstantCase() : 'ACTION_TYPE'),
      indent: (amount = 1) => this.indent.repeat(amount)
    };
  }

  addExports() {
    const destDirWithCreators = this.destDir.concat(CREATORS_FILE_NAME);
    let currentPath = `${this.topLevelPath}`;
    destDirWithCreators.forEach((subPath, index) => {
      const isLastLevel = index === destDirWithCreators.length - 1;
      const newCreatorExport = t.exportAllFrom(subPath); // read

      const indexFile = this.fs.read(`${currentPath}/index.js`, {
        defaults: ''
      }); // update

      const updatedIndexFile = (0, _utils.addToFile)(indexFile, newCreatorExport, r.selectExports); // write

      this.filesToWrite.push({
        name: `${currentPath}/index.js`,
        content: updatedIndexFile
      });

      if (this.withActionType && !isLastLevel) {
        const newTypesExport = t.exportAllFrom(`${subPath}/types`); // read

        const typesFile = this.fs.read(`${currentPath}/types.js`, {
          defaults: ''
        }); // update

        const updatedTypesFile = (0, _utils.addToFile)(typesFile, newTypesExport, r.selectExports); // write

        this.filesToWrite.push({
          name: `${currentPath}/types.js`,
          content: updatedTypesFile
        });
      }

      currentPath += `/${subPath}`;
    });
  }

  addActionCreator() {
    const creatorTemplate = this.fs.read(this.templatePath(`${PATH_PREFIX}/creator.ejs`));

    const newCreator = _ejs.default.render(creatorTemplate, this.templateConfig); // read


    const fileHead = t.importAllAsFrom('t', '@/actions/types') + '\n';
    const creatorsFile = this.fs.read(`${this.absolutePath}/creators.js`, {
      defaults: fileHead
    }); // update

    const updateOptions = {
      separator: '\n\n'
    };
    const updatedFile = (0, _utils.addToFile)(creatorsFile, newCreator, r.selectExports, updateOptions); // write

    this.filesToWrite.push({
      name: `${this.absolutePath}/creators.js`,
      content: updatedFile
    });
  }

  addActionCreatorTest() {
    const testTemplate = this.fs.read(this.templatePath(`${PATH_PREFIX}/creator.test.ejs`));

    const newTest = _ejs.default.render(testTemplate, this.templateConfig); // read


    const fileHead = t.importAllAsFrom('t', '@/actions/types') + t.importAllAsFrom('actions', './creators') + '\n' + t.describeTestStart(`actions/${this.destPath}`);
    const testFile = this.fs.read(`${this.absolutePath}/creators.test.js`, {
      defaults: fileHead
    }); // update

    const updateOptions = {
      appendixIfNew: t.describeTestEnd(),
      replaceStringSeparator: '  ',
      separator: '\n\n  '
    };
    const updatedFile = (0, _utils.addToFile)(testFile, newTest, r.selectDescribes, updateOptions); // write

    this.filesToWrite.push({
      name: `${this.absolutePath}/creators.test.js`,
      content: updatedFile
    });
  }

  addActionType() {
    if (this.withActionType) {
      const actionType = this.fileName.toConstantCase();
      const newExport = t.exportStringConst(actionType, `${this.destPath}/${actionType}`); // read

      const typesFile = this.fs.read(`${this.absolutePath}/types.js`, {
        defaults: ''
      }); // update

      const updatedFile = (0, _utils.addToFile)(typesFile, newExport, r.selectExports); // write

      this.filesToWrite.push({
        name: `${this.absolutePath}/types.js`,
        content: updatedFile
      });
    }
  }

  writing() {
    this.filesToWrite.forEach(({
      name,
      content
    }) => this.fs.write(name, `${content.trim()}\n`));
  }

}

exports.default = ActionGenerator;