"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _yeomanGenerator = _interopRequireDefault(require("yeoman-generator"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const PATH_PREFIX = 'components';

class ComponentGenerator extends _yeomanGenerator.default {
  constructor(args, options) {
    super(args, options);

    _defineProperty(this, "templateConfig", {});

    _defineProperty(this, "indent", void 0);

    _defineProperty(this, "destDir", void 0);

    _defineProperty(this, "destPath", void 0);

    _defineProperty(this, "topLevelPath", void 0);

    _defineProperty(this, "absolutePath", void 0);

    _defineProperty(this, "fileName", void 0);

    _defineProperty(this, "classComp", void 0);

    _defineProperty(this, "connected", false);

    _defineProperty(this, "withProps", true);

    this.indent = options.indent;
    this.destDir = options.destDir;
    this.destPath = this.destDir.join('/');
    this.topLevelPath = `${options.srcDir}/${PATH_PREFIX}`;
    this.absolutePath = `${this.topLevelPath}/${this.destPath}`;
    this.fileName = options.fileName;
    this.classComp = options.classComp === true;
  }

  async prompting() {
    const answer1 = await this.prompt({
      message: 'Do you want the component to be connected to the store?',
      name: 'connected',
      type: 'confirm'
    });
    this.connected = answer1.connected;

    if (!answer1.connected) {
      const answer2 = await this.prompt({
        message: 'Do you want the component to be set up with props?',
        name: 'withProps',
        type: 'confirm'
      });
      this.withProps = answer2.withProps;
    }
  }

  configuring() {
    this.templateConfig = {
      COMPONENT_NAME: this.fileName.toPascalCase(),
      CONNECTED: this.connected,
      WITH_PROPS: this.withProps,
      indent: (amount = 1) => this.indent.repeat(amount)
    };
  }

  writing() {
    const templateName = this.classComp ? 'classComp' : 'component';
    const componentName = this.fileName.toPascalCase();
    this.fs.copyTpl(this.templatePath(`${PATH_PREFIX}/${templateName}.ejs`), this.destinationPath(`${this.absolutePath}/${componentName}.js`), this.templateConfig);
  }

}

exports.default = ComponentGenerator;