"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _yeomanGenerator = _interopRequireDefault(require("yeoman-generator"));

var r = _interopRequireWildcard(require("../constants/regex"));

var t = _interopRequireWildcard(require("../constants/templateStrings"));

var _utils = require("../utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const PATH_PREFIX = 'reducers';

class ReducerGenerator extends _yeomanGenerator.default {
  constructor(args, options) {
    super(args, options);

    _defineProperty(this, "filesToWrite", []);

    _defineProperty(this, "templateConfig", {});

    _defineProperty(this, "indent", void 0);

    _defineProperty(this, "destDir", void 0);

    _defineProperty(this, "destPath", void 0);

    _defineProperty(this, "topLevelPath", void 0);

    _defineProperty(this, "absolutePath", void 0);

    _defineProperty(this, "fileName", void 0);

    this.indent = options.indent;
    this.destDir = options.destDir;
    this.destPath = this.destDir.join('/');
    this.topLevelPath = `${options.srcDir}/${PATH_PREFIX}`;
    this.absolutePath = `${this.topLevelPath}/${this.destPath}`;
    this.fileName = options.fileName;
  }

  initializing() {
    if (this.destDir.length === 0) {
      this.log('We do not support top-level reducers. Please specify at least one subdirectory.');
      process.exit(0);
    }
  }

  configuring() {
    this.templateConfig = {
      REDUCER_NAME: this.fileName,
      REDUCER_PATH: this.destDir.join('/'),
      SELECTOR_NAME: ['get', ...this.destDir].join('-').toCamelCase(),
      STATE_PARTS: this.destDir,
      STATE_PATH: this.destDir.join('.'),
      indent: (amount = 1) => this.indent.repeat(amount)
    };
  }

  addExports() {
    const destDirWithFileName = this.destDir.concat(this.fileName);
    let currentPath = `${this.topLevelPath}`;
    destDirWithFileName.forEach(subPath => {
      // read
      const fileHead = t.importNamedFrom('combineReducers', 'redux') + '\n';
      const file = this.fs.read(`${currentPath}/index.js`, {
        defaults: fileHead
      }); // update imports

      let updatedFile = (0, _utils.addToFile)(file, t.importDefaultFrom(subPath, `./${subPath}`), r.selectDefaultImports); // insert default export

      if (!r.selectCombineReducersMethod.test(file)) {
        const combineOptions = {
          prefixForAll: '\n'
        };
        updatedFile = (0, _utils.addToFile)(updatedFile, t.exportDefaultCombineReducers(), undefined, combineOptions);
      } // update combined reducers


      const updateOptions = {
        appendixIfNew: '});\n\n',
        prefixForAll: this.indent,
        separator: '',
        suffixForAll: '\n'
      };
      updatedFile = (0, _utils.addToFile)(updatedFile, `${subPath},`, r.selectCombinedReducers, updateOptions); // update selector exports

      updatedFile = (0, _utils.addToFile)(updatedFile, t.exportAllFrom(subPath), r.selectExportsAll);
      this.filesToWrite.push({
        name: `${currentPath}/index.js`,
        content: updatedFile
      });
      currentPath += `/${subPath}`;
    });
  }

  addReducer() {
    this.fs.copyTpl(this.templatePath(`${PATH_PREFIX}/file.ejs`), this.destinationPath(`${this.absolutePath}/${this.fileName}.js`), this.templateConfig);
  }

  addReducerTest() {
    this.fs.copyTpl(this.templatePath(`${PATH_PREFIX}/test.ejs`), this.destinationPath(`${this.absolutePath}/${this.fileName}.test.js`), this.templateConfig);
  }

  writing() {
    this.filesToWrite.forEach(({
      name,
      content
    }) => this.fs.write(name, `${content.trim()}\n`));
  }

}

exports.default = ReducerGenerator;