"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _ejs = _interopRequireDefault(require("ejs"));

var _fs = _interopRequireDefault(require("fs"));

var _fsReaddirRecursive = _interopRequireDefault(require("fs-readdir-recursive"));

var _fuzzy = _interopRequireDefault(require("fuzzy"));

var _inquirerAutocompletePrompt = _interopRequireDefault(require("inquirer-autocomplete-prompt"));

var _path = _interopRequireDefault(require("path"));

var _util = require("util");

var _yeomanGenerator = _interopRequireDefault(require("yeoman-generator"));

var r = _interopRequireWildcard(require("../constants/regex"));

var t = _interopRequireWildcard(require("../constants/templateStrings"));

var _logger = _interopRequireDefault(require("../logger"));

var _addToFile = _interopRequireDefault(require("../utils/addToFile"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const readFile = (0, _util.promisify)(_fs.default.readFile);
const PATH_PREFIX = 'selectors';
const REDUCER_PATH_PREFIX = 'reducers';
const POSSIBLE_STATE_NAMES = 'abcdefghijklmnopqrstuvwxyz'.split('');
const defaultExistingSelector = {
  source: 'reducers',
  path: '',
  name: '',
  displayName: ''
};

class SelectorGenerator extends _yeomanGenerator.default {
  constructor(args, options) {
    super(args, options);

    _defineProperty(this, "filesToWrite", []);

    _defineProperty(this, "templateConfig", {});

    _defineProperty(this, "indent", void 0);

    _defineProperty(this, "srcDir", void 0);

    _defineProperty(this, "destDir", void 0);

    _defineProperty(this, "destPath", void 0);

    _defineProperty(this, "topLevelPath", void 0);

    _defineProperty(this, "topLevelReducerPath", void 0);

    _defineProperty(this, "absolutePath", void 0);

    _defineProperty(this, "fileName", void 0);

    _defineProperty(this, "existingSelectors", []);

    _defineProperty(this, "chosenSelectors", []);

    this.indent = options.indent;
    this.srcDir = options.srcDir;
    this.destDir = options.destDir;
    this.destPath = this.destDir.join('/');
    this.topLevelPath = `${options.srcDir}/${PATH_PREFIX}`;
    this.topLevelReducerPath = `${options.srcDir}/${REDUCER_PATH_PREFIX}`;
    this.absolutePath = `${this.topLevelPath}/${this.destPath}`;
    this.fileName = options.fileName;
  }

  async initializing() {
    this.env.adapter.promptModule.registerPrompt('autocomplete', _inquirerAutocompletePrompt.default);
    const reducerFiles = (0, _fsReaddirRecursive.default)(this.topLevelReducerPath, name => !name.startsWith('.') && !name.endsWith('index.js') && !name.endsWith('test.js'));
    await Promise.all(reducerFiles.map(async filePath => {
      const file = await readFile(_path.default.resolve(this.topLevelReducerPath, filePath), 'utf8');
      const fileSelectors = file.match(r.selectExportConstNames) || [];
      fileSelectors.forEach(name => {
        const displayedPath = filePath.replace(/\/\w*\.js/, '').replace(/\//g, ' › ');
        this.existingSelectors.push({
          name,
          path: displayedPath,
          source: REDUCER_PATH_PREFIX,
          displayName: `${displayedPath} › ${name}`
        });
      });
    }));
  }

  async prompting() {
    const DONE_CMD = 'done!';

    const doneSelector = _objectSpread({}, defaultExistingSelector, {
      displayName: DONE_CMD
    });

    const answers = [];
    let remainingExistingSelectors = [...this.existingSelectors, doneSelector];

    _logger.default.selectorPrompt(DONE_CMD);

    const promptAnswer = async () => {
      const {
        selector
      } = await this.prompt({
        type: 'autocomplete',
        name: 'selector',
        message: '›',
        // autocomplete plugin requires this to be a promise
        source: async (_, input = '') => _fuzzy.default.filter(input, remainingExistingSelectors, {
          extract: s => s.displayName
        }).map(result => result.string)
      });

      if (selector && selector !== DONE_CMD) {
        answers.push(selector);
        remainingExistingSelectors = remainingExistingSelectors.filter(s => s.displayName !== selector);
        await promptAnswer();
      }
    };

    await promptAnswer();
    this.chosenSelectors = answers.map(answer => this.existingSelectors.find(({
      displayName
    }) => answer === displayName) || defaultExistingSelector);
  }

  configuring() {
    this.templateConfig = {
      SELECTOR_NAME: this.fileName,
      USED_SELECTORS: this.chosenSelectors.map(s => s.source === 'reducers' ? `s.${s.name}` : s.name),
      STATE_NAMES: POSSIBLE_STATE_NAMES.slice(0, this.chosenSelectors.length).join(', '),
      indent: (amount = 1) => this.indent.repeat(amount)
    };
  }

  addExports() {
    let currentPath = `${this.topLevelPath}`;
    this.destDir.forEach((subPath, index) => {
      // read
      const fileHead = index === 0 ? t.exportAllFromReducers() : '';
      const file = this.fs.read(`${currentPath}/index.js`, {
        defaults: fileHead
      }); // update

      const updatedFile = (0, _addToFile.default)(file, t.exportAllFrom(subPath), r.selectExportsAll); // write

      this.filesToWrite.push({
        name: `${currentPath}/index.js`,
        content: updatedFile
      });
      currentPath += `/${subPath}`;
    });
  }

  addSelector() {
    const selectorTemplate = this.fs.read(this.templatePath(`${PATH_PREFIX}/selector.ejs`));

    const newSelector = _ejs.default.render(selectorTemplate, this.templateConfig); // read


    const fileHead = t.importNamedFrom('createSelector', 'reselect') + '\n' + t.importAllAsFrom('s', '@/reducers') + '\n';
    const selectorsFile = this.fs.read(`${this.absolutePath}.js`, {
      defaults: fileHead
    }); // update

    const updateOptions = {
      separator: '\n\n'
    };
    const updatedFile = (0, _addToFile.default)(selectorsFile, newSelector, r.selectExports, updateOptions); // write

    this.filesToWrite.push({
      name: `${this.absolutePath}.js`,
      content: updatedFile
    });
  }

  addSelectorTest() {
    const testTemplate = this.fs.read(this.templatePath(`${PATH_PREFIX}/selector.test.ejs`));

    const newTest = _ejs.default.render(testTemplate, this.templateConfig); // read


    const fileHead = t.importAllAsFrom('s', `./${this.destPath}`) + '\n' + t.describeTestStart(`selectors/${this.destPath}`);
    const testFile = this.fs.read(`${this.absolutePath}.test.js`, {
      defaults: fileHead
    }); // update

    const updateOptions = {
      appendixIfNew: t.describeTestEnd(),
      replaceStringSeparator: '  ',
      separator: '\n\n  '
    };
    const updatedFile = (0, _addToFile.default)(testFile, newTest, r.selectDescribes, updateOptions);
    this.filesToWrite.push({
      name: `${this.absolutePath}.test.js`,
      content: updatedFile
    });
  }

  writing() {
    this.filesToWrite.forEach(({
      name,
      content
    }) => this.fs.write(name, `${content.trim()}\n`));
  }

}

exports.default = SelectorGenerator;