"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _chalk = _interopRequireDefault(require("chalk"));

var _generators = require("./generators");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// tslint:disable-next-line no-var-requires
const {
  version: pkgVersion
} = require('../package.json');

const {
  log
} = console;
const {
  bold,
  underline,
  gray
} = _chalk.default;
var _default = {
  log(text) {
    log(text);
  },

  version() {
    log(pkgVersion);
  },

  showHelpCmd() {
    log(`Use ${underline('zwen --help')} for more info.`);
  },

  help() {
    log(bold('USAGE'));
    log('  $ zwen [SCAFFOLD_TYPE] [PATH_WITH_NAME] [OPTIONS]');
    log('');
    log(bold('ARGUMENTS'));
    log(`  SCAFFOLD_TYPE     ${gray('What do you want to generate?')}`);
    log(`                    ${gray.underline(_generators.registeredGenerators.toString('|', '(', ')'))}`);
    log(`  PATH_WITH_NAME    ${gray('Where should the new scaffold be placed?')}`);
    log(`                    ${gray('for example ' + underline('data/users'))}`);
    log('');
    log(bold('OPTIONS'));
    log(`  -c, --classComp   ${gray('Will create a class component')}`);
    log(`                    ${gray('Useful if you need React lifecycle methods')}`);
    log(`  -h, --help        ${gray('Show this summary')}`);
    log(`  -v, --version     ${gray('Logs your installed ' + underline('zwen') + ' version')}`);
    log('');
    log(bold('EXAMPLES'));
    log(`  $ zwen reducer filter/date/selectedWeek`);
    log(`  $ zwen component ui/closeButton -c`);
  },

  unknownCommand(command) {
    log(`Unknown command: ${bold.red(command)}.`);
    log(`Available options are: ${bold.green(_generators.registeredGenerators.toString('|', '(', ')'))}.`);
    this.showHelpCmd();
  },

  missingArgument(arg) {
    log(`Missing argument: ${bold.red(arg)}`);
    this.showHelpCmd();
  },

  invalidConfig(option, comment) {
    log(`Your ${gray.underline('.zwen file')} contains an invalid configuration: ${bold.red(option)}.`);

    if (comment != null) {
      log(comment);
    }
  },

  selectorPrompt(command) {
    log(`Which selectors do you want to use?\nType ${gray.underline(command)} when you're done.`);
  }

};
exports.default = _default;